library ieee;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

-- M0 M1  SPI: master --> salve	PROG_B		disable_flash 	enable_flash
--  *  1  FX3 --> FLASH		1		hold		hold
--  0  0  FX3 --> FPGA		FPGA_RESET_N	0		1
--  1  0  FPGA -> FLASH		FPGA_RESET_N	1		0

entity configcpld is
   port(
      RESET_N_IN      	: in std_logic;  
      RESET_N        	: out std_logic;  

      FPGA_RESET_N   	: in std_logic;
      PROG_B     	: out std_logic; 

      MODE0		: in std_logic;
      MODE1		: in std_logic;

      JP1		: in std_logic;

      CLKIN        	: in std_logic;  
      FXCLK        	: out std_logic;  

      FX3_CS		: in std_logic;  
      FX3_CLK		: in std_logic;  
      FX3_MOSI		: in std_logic;  
      FX3_MISO		: out std_logic;  

      FPGA_CLK		: inout std_logic;  
      FPGA_CS		: inout std_logic;  
      FPGA_MOSI		: inout std_logic;  
      FPGA_MISO		: inout std_logic;  

      FLASH_CS		: out std_logic;  
      FLASH_CLK		: out std_logic;  
      FLASH_DIN		: out std_logic;  
      FLASH_DOUT	: in std_logic
   );
end configcpld;


architecture RTL of configcpld is

signal reset_release : std_logic := '0';
signal sdcounter : std_logic_vector(12 downto 0) := (others => '0');

signal disable_flash: std_logic := '0';
signal enable_flash: std_logic := '0';

begin
    PROG_B <= (FPGA_RESET_N or MODE1) and reset_release;
    FXCLK <= CLKIN;
    
    FPGA_CS <= FX3_CS when ( MODE1 ='0' and MODE0 ='0') else 'Z';
    FPGA_CLK <= FX3_CLK when ( MODE1 ='0' and MODE0 ='0') else 'Z';
    FPGA_MOSI <= FX3_MOSI when ( MODE1 ='0' and MODE0 ='0') else 'Z';
    FX3_MISO <= FPGA_MISO when (MODE1 = '0' and MODE0 = '0') else FLASH_DOUT;
    
    FLASH_CS <= ( FX3_CS or ((not enable_flash) and ((not JP1) or disable_flash)) ) when ( MODE1 = '1' )
    		else FPGA_CS when (MODE0 = '1') 
		else '1';

--  FLASH_CS <= FX3_CS or ( MODE1 and ((not JP1) or (disable_flash1 and disable_flash2)) );-- when ( MODE0 = MODE1 )
    FLASH_CLK <= FPGA_CLK when ( MODE1 = '0' and MODE0 = '1' ) else FX3_CLK;
    FLASH_DIN <= FPGA_MOSI when ( MODE1 = '0' and MODE0 = '1' ) else FX3_MOSI;
    FPGA_MISO <= FLASH_DOUT when ( MODE1 = '0' and MODE0 = '1' ) else 'Z';

--    FX3_MISO <= FLASH_DOUT; 

    reset_release <= '1' when sdcounter(12 downto 11) = "11" else '0';
    RESET_N <= reset_release;

    dpCLKIN: process(CLKIN)
    begin
        if CLKIN' event and CLKIN = '1' 
        then
	    if ( RESET_N_IN = '0' )
	    then
	        sdcounter(11) <= '0';
    	    end if;
    	    
	    if ( reset_release = '0' )
	    then
    		sdcounter <= sdcounter + 1;
	        disable_flash <= '0';
	        enable_flash <= '0';
	    else   
		if ( MODE0 = '0' and MODE1 = '0' )
		then 
	    	    disable_flash <= '0';
	    	    enable_flash <= '1';
	    	elsif ( MODE0 = '1' and MODE1 = '0' )
	    	then 
	    	    disable_flash <= '1';
	    	    enable_flash <= '0';
	    	end if;
		
    	    end if;
	end if;
    end process;
    
end RTL;

